# Developing Persefone

## Workflow

1. Pull the current version from the master branch on Gitlab: 
   [https://git.idiv.de/xo30xoqa/persephone](https://git.idiv.de/xo30xoqa/persephone).

2. If you are working on a new feature, create a new branch to avoid breaking the `master` branch.
   (The `master` branch on Github should always be in a runnable and error-free state.)

3. Implement your changes.

4. Run an example simulation and the test suite to make sure everything works without crashing
   (`make run` and `make test` on Linux, or execute `run.jl` and `test/runtests.jl` manually.)

5. Commit your work frequently, and try to keep each commit small. Don't forget to add
   relevant tests to the test suite.

6. Once your satisfied with your work, do another pull/merge from the `master` branch in case
   somebody else changed the branch in the meantime. Then merge your work into `master` and
   push to the Gitlab server.
   
8. Repeat :-)

The Gitlab [issue tracker](https://git.idiv.de/xo30xoqa/persephone/-/boards/373) can be used 
to create, discuss, and assign tasks, as well as to monitor progress towards milestones/releases. 
Once we have a first release, we will start using [semantic versioning](https://semver.org/).

## Libraries

### Agents.jl

Our model uses [Agents.jl](https://juliadynamics.github.io/Agents.jl/stable/) as a framework.
Their [repository](https://github.com/JuliaDynamics/Agents.jl) can be used to inspect the source
code or submit bug reports (the authors are quick to respond). Questions can be asked at the 
[Julia Discourse forum](https://discourse.julialang.org/c/domain/models/21).

*Tutorial on collaborating on Julia packages: 
[https://www.matecdev.com/posts/julia-package-collaboration.html](https://www.matecdev.com/posts/julia-package-collaboration.html).*

### Revise.jl

[`Revise.jl`](https://timholy.github.io/Revise.jl/stable/) allows one to reload code
without restarting the Julia interpreter. Get it with `Pkg.add("Revise")`, then 
add `using Revise` to `.julia/config/startup.jl` to have it automatically available.

### Test

Persefone uses the inbuilt Julia [testing framework](https://docs.julialang.org/en/v1/stdlib/Test/).
All new functions should have appropriate tests written for them in the appropriate
file in the `test` directory. (See [`test/runtests.jl`](https://git.idiv.de/xo30xoqa/persephone/-/blob/master/test/runtests.jl)
for details.) There are three ways to run the test suite: in the terminal, executing
`make test` or `cd test; julia runtests.jl`; or in the Julia REPL, 
`Pkg.activate("."); Pkg.test()`.

### Documenter.jl

The HTML documentation is generated using [Documenter.jl](https://documenter.juliadocs.org).
Therefore, all new functions should have docstrings attached. New files need to be integrated
into the relevant documentation source files in `docs/src`, and if necessary into
[`docs/builddocs.jl`](https://git.idiv.de/xo30xoqa/persephone/-/blob/master/docs/builddocs.jl).
To build the documentation, run `make docs`, or `cd docs; julia builddocs.jl`
(if using the latter, don't forget to update the date and commit in `docs/src/index.md`).

## Julia editors

### Emacs

There are a couple of addons that make working with Julia much nicer in Emacs:

1. `julia-mode` gives syntax highlighting. Install with `M-x package-install julia-mode`.

2. [`julia-snail`](https://github.com/gcv/julia-snail) provides IDE-like features, 
   especially a fully-functional REPL and the ability to evaluate code straight from inside a buffer.
   Note that the installation can be somewhat tricky. You first need to manually install
   all the dependencies of its dependency [vterm](https://github.com/akermu/emacs-libvterm),
   then install vterm itself with `M-x package-install vterm`, *before* you can do
   `M-x package-install julia-snail`. Then add it to your `init.el` with
   `(require 'julia-snail)` and `(add-hook 'julia-mode-hook #'julia-snail-mode)`.
   
3. [`company-mode`](http://company-mode.github.io/) integrates with Snail to give code 
   completion. Install with `M-x package-install company`, then add 
   `(add-hook 'julia-mode-hook #'company-mode)` and 
   `(global-set-key (kbd "C-<tab>") 'company-complete)` to your `init.el`.
   
4. [`magit`](https://magit.vc/) is a great git interface for Emacs. Install with
   `M-x package-install magit` and add `(global-set-key (kbd "C-x g") 'magit-status)`
   to your `init.el`.

### VSCode

See [here](https://www.julia-vscode.org/).

*TODO: add more detail.*

